﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.ComponentModel;

namespace CSharpExample.Common
{
    public class Async
    {
        public static void ExecAsync(Form form, Action action, Action onComplete)
        {
            if (form.Controls.ContainsKey("LongOperationControl"))
                return;

            var backgroundWorker = new BackgroundWorker();

            var txt = form.Text;

            var dic = new Dictionary<string, bool>();

            var focus = FindFocused(form);

            foreach (Control item in form.Controls)
            {
                dic.Add(item.Name, item.Enabled);
                item.Enabled = false;
            }

            var lc = new LongOperationControl();
            lc.Name = "LongOperationControl";
            lc.Location =
                new System.Drawing.Point(
                    form.Size.Width / 2 - lc.Size.Width / 2,
                    form.Size.Height / 2 - lc.Size.Height / 2
                    );


            form.Resize += form_Resize;

            form.Controls.Add(lc);
            lc.BringToFront();


            backgroundWorker.DoWork += (sender, e) => action();
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.ProgressChanged += (sender, args) =>
            {
                var msg = args.UserState.ToString();
                lc.ProgressMessage = msg;
            };

            //backgroundWorker.ProgressChanged += (sender, e) => action();
            backgroundWorker.RunWorkerCompleted += (sender, e) =>
            {
                form.Text = txt;
                form.Resize -= form_Resize;

                //form.Controls.Remove(lc);
                form.Controls.RemoveByKey("LongOperationControl");

                foreach (Control item in form.Controls)
                {
                    item.Enabled = dic[item.Name];
                    //item.Enabled = true;
                }

                if (e.Error != null)
                {

                    throw e.Error;
                }

                if (onComplete != null)
                    onComplete();

                if (focus != null)
                    focus.Focus();
            };



            form.Text = txt + "~";
            backgroundWorker.RunWorkerAsync();
        }

        public static void ExecAsync(Form form, Action<BackgroundWorker> action, Action onComplete)
        {
            if (form.Controls.ContainsKey("LongOperationControl"))
                return;

            var backgroundWorker = new BackgroundWorker();
            
            var txt = form.Text;

            var dic = new Dictionary<string, bool>();

            var focus = FindFocused(form);

            foreach (Control item in form.Controls)
            {
                dic.Add(item.Name, item.Enabled);
                item.Enabled = false;
            }

            var lc = new LongOperationControl();
            lc.Name = "LongOperationControl";
            lc.Location =
                new System.Drawing.Point(
                    form.Size.Width / 2 - lc.Size.Width / 2,
                    form.Size.Height / 2 - lc.Size.Height / 2
                    );
            

            form.Resize += form_Resize;

            form.Controls.Add(lc);
            lc.BringToFront();
            

            backgroundWorker.DoWork += (sender, e) => action(backgroundWorker);
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.ProgressChanged += (sender, args) =>
                                                    {
                                                        var msg = args.UserState.ToString();
                                                        lc.ProgressMessage = msg;
                                                    };

            //backgroundWorker.ProgressChanged += (sender, e) => action();
            backgroundWorker.RunWorkerCompleted += (sender, e) =>
            {
                form.Text = txt;
                form.Resize -= form_Resize;

                //form.Controls.Remove(lc);
                form.Controls.RemoveByKey("LongOperationControl");

                foreach (Control item in form.Controls)
                {
                    item.Enabled = dic[item.Name];
                    //item.Enabled = true;
                }

                if (e.Error != null)
                {
                    
                    throw e.Error;
                }

                if (onComplete != null)
                    onComplete();

                if (focus != null)
                    focus.Focus();
            };

            

            form.Text = txt + "~";
            backgroundWorker.RunWorkerAsync();
        }

        public static void ExecAsyncSilent(Form form, Action<BackgroundWorker> action, Action onComplete)
        {
            var backgroundWorker = new BackgroundWorker();

            var txt = form.Text;

            backgroundWorker.DoWork += (sender, e) => action(backgroundWorker);
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.ProgressChanged += (sender, args) =>
            {
                var msg = args.UserState.ToString();
                //lc.ProgressMessage = msg;
            };

            backgroundWorker.RunWorkerCompleted += (sender, e) =>
            {
                form.Text = txt;
                if (e.Error != null)
                {
                    
                    throw e.Error;
                }

                if (onComplete != null)
                    onComplete();

                
            };

            form.Text = txt + "~";
            backgroundWorker.RunWorkerAsync();
        }

        static void form_Resize(object sender, EventArgs e)
        {
            var form = ((Form)sender);
            var lc = form.Controls["LongOperationControl"];
            lc.Location =
                new System.Drawing.Point(
                    form.Size.Width / 2 - lc.Size.Width / 2,
                    form.Size.Height / 2 - lc.Size.Height / 2
                    );
            
        }

        static Control FindFocused(Control form)
        {
            Control res = null;
            foreach (Control control in form.Controls)
            {
                if (control.Focused)
                    res = control;
                else
                    res = FindFocused(control);

                if (res != null)
                    break;
            }

            return res;
        }
    }
}
