﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;

namespace CSharpExample
{
    public class Deal
    {
        public Deal()
        {
        }
        
        public virtual string ExCode { get; set; }
        public virtual string Agreement { get; set; }
        public virtual string Reference { get; set; }
        public virtual DateTime? TradeDate { get; set; }
        public virtual string Participant { get; set; }
        public virtual string Type { get; set; }
        public virtual string InName { get; set; }
        public virtual string OnAccount { get; set; }
        public virtual string Issue { get; set; }
        public virtual string Isin { get; set; }
        public virtual string RegNum { get; set; }
        public virtual decimal? Qty { get; set; }
        public virtual decimal? QtyFrac { get; set; }
        public virtual decimal? Price { get; set; }
        public virtual string Currency { get; set; }
        public virtual string SettlCurrency { get; set; }
        public virtual decimal? Rate { get; set; }
        public virtual string Block { get; set; }
        public virtual string Settle { get; set; }
        public virtual DateTime? SettleDate { get; set; }

        
        public virtual int? DatabaseId { get; set; }
        public virtual string Errors { get; set; }
        public virtual string Warnings { get; set; }

        public static DealEnvelope GetPostRequest(Deal[] deals, bool includeDealId)
        {
            var env = new DealEnvelope();
            env.Id = Guid.NewGuid().ToString("N");
            env.Language = "RU";
            env.CustomRef = "";

            foreach (var deal in deals)
            {
                var dic = new Dictionary<string, string>();
                var p = typeof(Deal).GetProperties();
                foreach (var info in p)
                {
                    if (info.PropertyType == typeof(DateTime?))
                    {
                        var val = (DateTime?)info.GetValue(deal, new object[] { }) ?? (DateTime?)null;
                        dic.Add(info.Name, val.HasValue ? val.Value.ToString("yyyy-MM-dd") : "");
                    }
                    else
                    {
                        dic.Add(info.Name, (info.GetValue(deal, new object[] { }) ?? (object)"").ToString());
                    }
                }

                env.Deals.Add(dic);
            }

            return env;
        }

        public static DealEnvelope GetDeleteRequest(Deal[] deals, string reason)
        {
            var env = new DealEnvelope();
            env.Id = Guid.NewGuid().ToString("N");
            env.Language = "RU";
            env.CustomRef = "";

            foreach (var deal in deals)
            {
                var dic = new Dictionary<string, string>();

                dic.Add("Id", deal.DatabaseId.ToString());
                dic.Add("Agreement", deal.Agreement);
                dic.Add("Reference", deal.Reference);
                dic.Add("Participant", deal.Participant);
                dic.Add("RevokeReason", reason);
                env.Deals.Add(dic);
            }

            return env;
        }
        public static Deal FromDictionary(Dictionary<string, string> item)
        {
            var deal = new Deal();
            var properties = typeof(Deal).GetProperties();

            foreach (var info in properties)
            {
                if (item.ContainsKey(info.Name))
                {
                    var value = item[info.Name];
                    if (info.PropertyType == typeof(DateTime?))
                    {
                        if (value == "")
                        {
                            info.SetValue(deal, null, new object[] { });
                        }
                        else
                        {
                            var dt = DateTime.ParseExact(value, "yyyy-MM-dd", CultureInfo.InvariantCulture);
                            info.SetValue(deal, dt, new object[] { });
                        }
                    }
                    else if (info.PropertyType == typeof(decimal?))
                    {
                        if (value == "")
                        {
                            info.SetValue(deal, null, new object[] { });
                        }
                        else
                        {
                            var dt = decimal.Parse(value, CultureInfo.InvariantCulture);
                            info.SetValue(deal, dt, new object[] { });
                        }
                    }
                    else if (info.PropertyType == typeof(int?))
                    {
                        if (value == "")
                        {
                            info.SetValue(deal, null, new object[] { });
                        }
                        else
                        {
                            var dt = int.Parse(value);
                            info.SetValue(deal, dt, new object[] { });
                        }
                    }
                    else if (info.PropertyType == typeof(Guid))
                    {
                        var g = Guid.Parse(value);
                        info.SetValue(deal, g, new object[] { });
                    }
                    else
                    {
                        info.SetValue(deal, value, new object[] { });
                    }
                }
            }

            return deal;
        }
    }
}