﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Web.Script.Serialization;
using System.Windows.Forms;
using CSharpExample.Common;
using CSharpExample.Lists;

namespace CSharpExample
{
    public partial class MainForm : Form
    {
        private readonly SynchronizationContext _sync;
        public MainForm()
        {
            InitializeComponent();
            _sync = SynchronizationContext.Current;
            tbUrl.SelectedIndex = 0;
            tbPassportUrl.SelectedIndex = 0;

            dtFrom.Value = DateTime.Now.Date.AddMonths(-6);
            dtTo.Value = DateTime.Now.Date;
        }

        private void btnAuth_Click(object sender, EventArgs e)
        {
            Async.ExecAsync(this, Auth, null);
        }

        private CookieContainer cookiejar;
        private Cookie passport;
        public static CookieContainer CookieContainer { get; private set; }

        void Auth()
        {
            try
            {



                var url = "";
                _sync.Send((o)=>url=tbPassportUrl.Text, null);
                var usrStr = "";
                _sync.Send((o)=>usrStr=tbUser.Text + ":" + tbPass.Text, null);

                var servUrl = "";
                _sync.Send((o)=>servUrl = tbUrl.Text, null);

                HttpWebRequest AuthReq = (HttpWebRequest)WebRequest.Create(url);
                HttpWebResponse AuthResponse;
                AuthReq.CookieContainer = new CookieContainer();

                byte[] binData;
                binData = System.Text.Encoding.UTF8.GetBytes(usrStr);
                var sAuth64 = Convert.ToBase64String(binData, System.Base64FormattingOptions.None);
                AuthReq.Headers.Add(HttpRequestHeader.Authorization, "Basic " + sAuth64);

                AuthResponse = (HttpWebResponse)AuthReq.GetResponse();
                AuthResponse.Close();


                cookiejar = new CookieContainer();
                cookiejar.Add(AuthResponse.Cookies);

                // find the Passport cookie for a given domain URI
                var myuri = new System.Uri(url);

                passport = new Cookie();
                foreach (Cookie cook in cookiejar.GetCookies(myuri))
                {
                    if (cook.Name == "MicexPassportCert")
                    {
                        passport = cook;
                    }
                }



                if (passport.Name != "MicexPassportCert")
                {
                    _sync.Send((o)=>label1.Text = "Passport cookie not found", null);
                }
                else
                {
                    //*
                    //var localCook = new Cookie(passport.Name, passport.Value, passport.Path, "127.0.0.1");
                    var domain = new Uri(servUrl);

                    passport.Domain = domain.Host;// "127.0.0.1";
                    cookiejar = new CookieContainer();
                    cookiejar.Add(passport);
                    //*/
                    CookieContainer = cookiejar;

                    _sync.Send((o)=>label1.Text = "Ok", null);

                }

            }
            catch (WebException ex)
            {
                Log(ex.Message);
                if (ex.Status == WebExceptionStatus.ProtocolError)
                {
                    Log(string.Format("Status Code : {0}", (ex.Response as HttpWebResponse).StatusCode));
                    Log(string.Format("Status Description : {0}", (ex.Response as HttpWebResponse).StatusDescription));
                }

            }
            catch (Exception ex)
            {
                Log(ex.Message);
            }
        }

        private void Log(string message)
        {
            _sync.Send((o)=>tbLog.Text += message + Environment.NewLine, null);
        }

        private void btnGet_Click(object sender, EventArgs e)
        {
            Async.ExecAsync(this, GetData, null); 
        }

        string Get()
        {

            var url ="";
            _sync.Send((o)=>url= tbUrl.Text, null);

            _sync.Send((o)=>url += "?DateFrom=" + dtFrom.Value.ToString("yyyy-MM-dd"), null);
            _sync.Send((o)=>url += "&DateTo=" + dtTo.Value.ToString("yyyy-MM-dd"), null);

            var getRequest = (HttpWebRequest)WebRequest.Create(url);
            getRequest.Method = "GET";
            getRequest.CookieContainer = cookiejar;

            var getResponse = (HttpWebResponse)getRequest.GetResponse();
            Stream getResponseStream = getResponse.GetResponseStream();

            var sr = new StreamReader(getResponseStream);

            return sr.ReadToEnd();
        }

        void GetData()
        {
           
            var list = new List<Deal>();
            _sync.Send((o)=>gvDeals.DataSource = null, null);

            var data = Get();
            try
            {
                var serializer = new JavaScriptSerializer();


                serializer.MaxJsonLength = int.MaxValue;


                if (data.StartsWith("{\"server_error"))
                {
                    var dic = serializer.DeserializeObject(data) as Dictionary<string, object>;
                    throw new Exception(dic["server_error"].ToString());
                }

                var res = serializer.Deserialize<DealEnvelope>(data);


                foreach (var deal in res.Deals)
                {
                    var d = Deal.FromDictionary(deal);
                    list.Add(d);
                }
                _sync.Send((o)=>lblTotal.Text = "всего " + res.TotalPages + " страниц", null);

                _sync.Send((o)=>gvDeals.DataSource = list, null);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }



        private void gvDeals_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.RowIndex >= 0 && e.ColumnIndex >= 0)
            {
                var selectedRow = gvDeals.Rows[e.RowIndex];
                var deal = selectedRow.DataBoundItem as Deal;

                if (deal != null)
                {
                    var f = new DealProcessingForm(deal, tbUrl.Text, cookiejar);
                    f.ShowDialog();
                }
            }
        }

        private void gvDeals_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            var dgv = (DataGridView)sender;
            var colName = dgv.Columns[e.ColumnIndex].Name;

            var lists = new AllLists();
            if (lists[colName].Count != 0)
            {
                e.Value = lists[colName].Find(value => value.Key == (string)e.Value).Value;
                e.FormattingApplied = true;
            }
        }


    }
}
