package com.moex;

import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;

import java.io.*;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

public class Sample {

    private static final String COOKIE = "Cookie";
    private static final String CONTENT_DISPOSITION = "Content-Disposition";
    private static final String FILENAME = "filename";
    private static final String SEPARATOR = "==";

    private static final String COOKIE_PROPERTY_NAME = "Set-Cookie";

    private static final String CERT_NAME = "MicexPassportCert";
    private static String cert;

    private static String login;
    private static String password;
    private static final String AUTHORIZATION_URL_NAME = "authorizationURL";
    private static String authorizationURL = "https://passport.beta.micex.com/authenticate";
    private static final String MAIN_URL_NAME = "mainURL";
    private static String mainURL = "http://dev.api.otc.cabinet.moex.com";
    private static final String METHOD_TYPE_NAME = "methodTypeName";
    private static Method methodType;
    private static final String REQUEST_NAME = "request";
    private static String request;
    private static final String OUTPUT_FILE_NAME = "outputFileName";
    private static FileOutputStream outputStream;
    private static StringBuilder errorText = new StringBuilder();
    private static Console console = System.console();

    private enum Method {
        DELETE("DELETE"),
        POST("POST"),
        GET("GET");

        private String methodName;

        Method(String methodName) {
            this.methodName = methodName;
        }

        public String getMethodName() {
            return methodName;
        }

        public static Method findByName(String methodName) {
            for (Method method : values()) {
                if (method.getMethodName().equals(methodName)) {
                    return method;
                }
            }
            return null;
        }
    }

    public static void main(String[] args) throws IOException {
        try {
            parseInputParams(args);
            auth();
            if (Method.GET.equals(methodType)) {
                sendGet(mainURL);
            } else {
                sendMessage(methodType, mainURL, request);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private static void parseInputParams(String[] args) {
        Map<String, String> inputParams = createInputParamsMap(args);
        authorizationURL = parseStringParam(inputParams, AUTHORIZATION_URL_NAME, "Invalid URL for authorization");
        mainURL = parseStringParam(inputParams, MAIN_URL_NAME, "Invalid URL for call service");
        parseMethodType(inputParams);
        parseRequest(inputParams);
        parseOutputFileName(inputParams);
        if (errorText.length() > 0) {
            addInputParamManual();
            console.printf(errorText.toString());
            System.exit(1);
        }
    }

    private static void addInputParamManual() {
        errorText.append("Input param format:\n");
        errorText.append(AUTHORIZATION_URL_NAME);
        errorText.append(SEPARATOR);
        errorText.append("URL for user authorization;\n");
        errorText.append(MAIN_URL_NAME);
        errorText.append(SEPARATOR);
        errorText.append("URL for call service;\n");
        errorText.append(METHOD_TYPE_NAME);
        errorText.append(SEPARATOR);
        errorText.append("request type (POST, GET or DELETE);\n");
        errorText.append(OUTPUT_FILE_NAME);
        errorText.append(SEPARATOR);
        errorText.append("file name for write response of service;\n");
        errorText.append(REQUEST_NAME);
        errorText.append(SEPARATOR);
        errorText.append("request params (not required);\n");
    }

    private static Map<String, String> createInputParamsMap(String[] args) {
        Map<String, String> inputParams = new HashMap<String, String>();
        for (String arg : args) {
            String[] parts = arg.trim().split(SEPARATOR);
            if (parts.length == 2) {
                inputParams.put(parts[0], parts[1]);
            }
        }
        return inputParams;
    }

    private static String parseStringParam(Map<String, String> inputParams, String paramName, String errorMessage) {
        String paramValue = inputParams.get(paramName);
        if (paramValue == null || paramValue.isEmpty()) {
            errorText.append(errorMessage);
            errorText.append("\n");
        }
        return paramValue;
    }

    private static void parseMethodType(Map<String, String> inputParams) {
        methodType = Method.findByName(inputParams.get(METHOD_TYPE_NAME));
        if (methodType == null) {
            errorText.append("Invalid request type\n");
        }
    }

    private static void parseRequest(Map<String, String> inputParams) {
        request = inputParams.get(REQUEST_NAME);
        if (!Method.GET.equals(methodType) && (request == null || request.isEmpty())) {
            errorText.append("Invalid request params\n");
        }
    }

    private static void parseOutputFileName(Map<String, String> inputParams) {
        String fileName = inputParams.get(OUTPUT_FILE_NAME);
        if (fileName == null || fileName.isEmpty()) {
            errorText.append("Not found file name for write responce of service\n");
            return;
        }
        try {
            outputStream = new FileOutputStream(fileName);
        } catch (FileNotFoundException e) {
            errorText.append("Can't create/open file ");
            errorText.append(fileName);
            errorText.append("\n");
        }
    }

    private static void auth() throws IOException {
        enterAuthData();
        URL url = new URL(authorizationURL);
        CredentialsProvider credsProvider = new BasicCredentialsProvider();
        credsProvider.setCredentials(
                new AuthScope(url.getHost(), url.getPort() == -1 ? url.getDefaultPort() : url.getPort()),
                new UsernamePasswordCredentials(login, password));
        CloseableHttpClient httpClient = HttpClients.custom()
                .setDefaultCredentialsProvider(credsProvider).build();
        HttpGet httpGet = new HttpGet(authorizationURL);
        CloseableHttpResponse response = null;
        try {
            response = httpClient.execute(httpGet);
            Header[] headers = response.getHeaders(COOKIE_PROPERTY_NAME);
            if (headers != null) {
                for (Header header : headers) {
                    for (HeaderElement element : header.getElements()) {
                        if (CERT_NAME.equals(element.getName())) {
                            cert = element.toString();
                            console.printf("Authentication is successful\n");
                            return;
                        }
                    }
                }
            }
            if (cert == null || cert.isEmpty()) {
                throw new IllegalStateException("Passport cookie not found");
            }
        } catch (IOException e) {
            throw new IllegalStateException("Passport cookie not found");
        } finally {
            if (response != null) {
                response.close();
            }
            httpClient.close();
        }
    }

    private static void enterAuthData() {
        console.printf("Enter your username: ");
        login = console.readLine();
        console.printf("Enter your password: ");
        password = new String(console.readPassword());
    }

    public static void sendMessage(Method method, String targetURL, String urlParameters) throws IOException {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            HttpEntityEnclosingRequestBase httpMethod = getHttpMethod(method, targetURL);
            StringEntity input = new StringEntity(urlParameters, ContentType.APPLICATION_JSON);
            httpMethod.setEntity(input);
            httpMethod.setHeader(new BasicHeader(COOKIE, cert));
            CloseableHttpResponse response = httpClient.execute(httpMethod);
            try {
                HttpEntity entity = response.getEntity();
                if (entity != null) {
                    entity.writeTo(outputStream);
                }
            } finally {
                response.close();
            }
        } finally {
            httpClient.close();
        }
    }

    private static HttpEntityEnclosingRequestBase getHttpMethod(Method method, String url) {
        if (Method.DELETE.equals(method)) {
            return new HttpDeleteWithBody(url);
        }
        return new HttpPost(url);
    }

    public static void sendGet(String targetURL) throws IOException {
        String fileName;
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            HttpGet httpGet = new HttpGet(targetURL);
            httpGet.setHeader(new BasicHeader(COOKIE, cert));
            CloseableHttpResponse response = httpClient.execute(httpGet);
            fileName = saveFile(response);
            if (fileName == null) {
                try {
                    HttpEntity entity = response.getEntity();
                    if (entity != null) {
                        entity.writeTo(outputStream);
                    }
                } finally {
                    response.close();
                }
            }
        } finally {
            httpClient.close();
        }
        if (fileName != null) {
            outputStream.write(fileName.getBytes());
        }
    }

    private static String saveFile(CloseableHttpResponse response) throws IOException {
        String fileName = null;
        for (Header header : response.getAllHeaders()) {
            if (CONTENT_DISPOSITION.equals(header.getName())) {
                for (HeaderElement element : header.getElements()) {
                    for (NameValuePair pair : element.getParameters()) {
                        if (FILENAME.equals(pair.getName())) {
                            fileName = pair.getValue();
                            response.getEntity().writeTo(new FileOutputStream(fileName));
                        }
                    }
                }
            }
        }
        return fileName;
    }

    static class HttpDeleteWithBody extends HttpEntityEnclosingRequestBase {

        @Override
        public String getMethod() {
            return Method.DELETE.getMethodName();
        }

        public HttpDeleteWithBody(final String uri) {
            super();
            setURI(URI.create(uri));
        }
    }
}